#!/usr/bin/env python
# -*- coding: iso-8859-15 -*-
#------------------------------------------------------------------------
# Application :    Noethys, gestion multi-activits
# Site internet :  www.noethys.com
# Auteur:           Ivan LUCAS
# Copyright:       (c) 2010-16 Ivan LUCAS
# Licence:         Licence GNU GPL
#------------------------------------------------------------------------

import Chemins
import os
import sys
import shutil
import platform
import subprocess
import UTILS_Customize
import appdirs


def GetRepData(fichier=""):
    # Vrifie si un rpertoire 'Portable' existe
    chemin = Chemins.GetMainPath("Portable")
    if os.path.isdir(chemin):
        chemin = os.path.join(chemin, "Data")
        if not os.path.isdir(chemin):
            os.mkdir(chemin)
        return os.path.join(chemin, fichier)

    # Recherche s'il existe un chemin personnalis dans le Customize.ini
    chemin = UTILS_Customize.GetValeur("repertoire_donnees", "chemin", "")
    #chemin = chemin.decode("iso-8859-15")
    if chemin != "" and os.path.isdir(chemin):
        return os.path.join(chemin, fichier)

    # Recherche le chemin du rpertoire des donnes
    if sys.platform == "win32" and platform.release() != "Vista" :

        chemin = appdirs.site_data_dir(appname=None, appauthor=False)
        #chemin = chemin.decode("iso-8859-15")

        chemin = os.path.join(chemin, "noethys")
        if not os.path.isdir(chemin):
            os.mkdir(chemin)

    else :

        chemin = appdirs.user_data_dir(appname=None, appauthor=False)
        #chemin = chemin.decode("iso-8859-15")

        chemin = os.path.join(chemin, "noethys")
        if not os.path.isdir(chemin):
            os.mkdir(chemin)

        chemin = os.path.join(chemin, "Data")
        if not os.path.isdir(chemin):
            os.mkdir(chemin)

    # Ajoute le dirname si besoin
    return os.path.join(chemin, fichier)


def GetRepTemp(fichier=""):
    chemin = GetRepUtilisateur("Temp")
    return os.path.join(chemin, fichier)

def GetRepUpdates(fichier=""):
    chemin = GetRepUtilisateur("Updates")
    return os.path.join(chemin, fichier)

def GetRepLang(fichier=""):
    chemin = GetRepUtilisateur("Lang")
    return os.path.join(chemin, fichier)

def GetRepSync(fichier=""):
    chemin = GetRepUtilisateur("Sync")
    return os.path.join(chemin, fichier)

def GetRepUtilisateur(fichier=""):
    """ Recherche le rpertoire Utilisateur pour stockage des fichiers de config et provisoires """
    chemin = None

    # Vrifie si un rpertoire 'Portable' existe
    chemin = Chemins.GetMainPath("Portable")
    if os.path.isdir(chemin):
        return os.path.join(chemin, fichier)

    # Recherche le chemin du rpertoire de l'utilisateur
    chemin = appdirs.user_config_dir(appname=None, appauthor=False, roaming=True)
    #chemin = chemin.decode("iso-8859-15")

    # Ajoute 'noethys' dans le chemin et cration du rpertoire
    chemin = os.path.join(chemin, "noethys")
    if not os.path.isdir(chemin):
        os.mkdir(chemin)

    # Ajoute le dirname si besoin
    return os.path.join(chemin, fichier)

def DeplaceFichiers():
    """ Vrifie si des fichiers du rpertoire Data ou du rpertoire Utilisateur sont  dplacer vers le rpertoire Utilisateur>AppData>Roaming """

    # Dplace les fichiers de config et le journal
    for nom in ("journal.log", "Config.dat", "Customize.ini") :
        for rep in ("", Chemins.GetMainPath("Data"), os.path.join(os.path.expanduser("~"), "noethys")) :
            fichier = os.path.join(rep, nom)
            if os.path.isfile(fichier) :
                print ["deplacement fichier config :", fichier, " > ", GetRepUtilisateur(nom)]
                shutil.move(fichier, GetRepUtilisateur(nom))

    # Dplace les fichiers xlang
    if os.path.isdir(Chemins.GetMainPath("Lang")) :
        for nomFichier in os.listdir(Chemins.GetMainPath("Lang")) :
            if nomFichier.endswith(".xlang") :
                print ["deplacement fichier xlang :", fichier, " > ", GetRepLang(nomFichier)]
                shutil.move(u"Lang/%s" % nomFichier, GetRepLang(nomFichier))

    # Dplace les fichiers du rpertoire Sync
    if os.path.isdir(Chemins.GetMainPath("Sync")) :
        for nomFichier in os.listdir(Chemins.GetMainPath("Sync")) :
            shutil.move(Chemins.GetMainPath("Sync/%s" % nomFichier), GetRepSync(nomFichier))

    # Dplace les fichiers de donnes du rpertoire Data
    if GetRepData() != "Data/" and os.path.isdir(Chemins.GetMainPath("Data")) :
        for nomFichier in os.listdir(Chemins.GetMainPath("Data")) :
            nomFichier = nomFichier.decode("iso-8859-15")
            if nomFichier.endswith(".dat") and "_" in nomFichier and "EXEMPLE_" not in nomFichier and "_archive.dat" not in nomFichier :
                # Dplace le fichier vers le rpertoire des fichiers de donnes
                print ["copie base de donnees :", nomFichier, " > ", GetRepData(nomFichier)]
                shutil.copy(Chemins.GetMainPath(u"Data/%s" % nomFichier), GetRepData(nomFichier))
                # Renomme le fichier de donnes en archive (par scurit)
                try :
                    os.rename(Chemins.GetMainPath(u"Data/%s" % nomFichier), Chemins.GetMainPath(u"Data/%s" % nomFichier.replace(".dat", "_archive.dat")))
                except :
                    pass

def DeplaceExemples():
    """ Dplace les fichiers exemples vers le rpertoire des fichiers de donnes """
    if GetRepData() != "Data/" :
        chemin = Chemins.GetStaticPath("Exemples")
        for nomFichier in os.listdir(chemin) :
            if nomFichier.endswith(".dat") and "EXEMPLE_" in nomFichier :
                # Dplace le fichier vers le rpertoire des fichiers de donnes
                shutil.copy(os.path.join(chemin, nomFichier), GetRepData(nomFichier))

def OuvrirRepertoire(rep):
    if platform.system() == "Windows":
        subprocess.Popen(["explorer", rep])
    elif platform.system() == "Darwin":
        subprocess.Popen(["open", rep])
    else:
        subprocess.Popen(["xdg-open", rep])



if __name__ == "__main__":
    # Teste les dplacements de fichiers
    # DeplaceFichiers()

    # Rpertoire utilisateur
    print GetRepUtilisateur()

    # Rpertoire des donnes
    chemin = GetRepData()
    print 1, os.path.join(chemin, u"Test.pdf")
    print 2, os.path.join(chemin, "Test.pdf")